@@ -50,7 +50,6 @@ from gajim.common.events import AllowGajimUpdateCheck
 from gajim.common.events import GajimUpdateAvailable
 from gajim.common.events import SignedIn
 from gajim.common.helpers import from_one_line
-from gajim.common.helpers import get_global_show
 from gajim.common.helpers import get_random_string
 from gajim.common.settings import LegacyConfig
 from gajim.common.settings import Settings
@@ -76,11 +75,11 @@ class CoreApplication(ged.EventHelper):
         app.detect_dependencies()
         configpaths.create_paths()
 
-        passwords.init()
-
         app.settings = Settings()
         app.settings.init()
 
+        passwords.init()
+
         app.config = LegacyConfig()
         app.commands = ChatCommands()
 
@@ -337,10 +336,10 @@ class CoreApplication(ged.EventHelper):
                        username: str,
                        domain: str,
                        password: str,
-                       proxy_name: str,
+                       proxy_name: str | None,
                        custom_host: tuple[str,
                                           ConnectionProtocol,
-                                          ConnectionType],
+                                          ConnectionType] | None,
                        anonymous: bool = False
                        ) -> None:
 
@@ -427,9 +426,6 @@ class CoreApplication(ged.EventHelper):
                       account: Optional[str] = None
                       ) -> None:
 
-        if status is None:
-            status = get_global_show()
-
         if account is not None:
             self._change_status(account, status)
             return
