@@ -16,36 +16,35 @@ from __future__ import annotations
 
 import typing
 from typing import Any
-from typing import Union
-from typing import Optional
 from typing import Callable
+from typing import Optional
+from typing import Union
 
 from dataclasses import dataclass
 from dataclasses import field
 
-from nbxmpp.protocol import JID
-from nbxmpp.modules.security_labels import Catalog
-from nbxmpp.modules.security_labels import Displaymarking
-from nbxmpp.modules.security_labels import SecurityLabel
-from nbxmpp.structs import HTTPAuthData
-from nbxmpp.structs import ModerationData
-from nbxmpp.structs import LocationData
-from nbxmpp.structs import RosterItem
-from nbxmpp.structs import TuneData
 from nbxmpp.const import Affiliation
 from nbxmpp.const import InviteType
 from nbxmpp.const import Role
 from nbxmpp.const import StatusCode
+from nbxmpp.modules.security_labels import Catalog
+from nbxmpp.modules.security_labels import Displaymarking
+from nbxmpp.modules.security_labels import SecurityLabel
+from nbxmpp.protocol import JID
+from nbxmpp.structs import HTTPAuthData
+from nbxmpp.structs import LocationData
+from nbxmpp.structs import ModerationData
+from nbxmpp.structs import RosterItem
+from nbxmpp.structs import TuneData
 
-from gajim.common.file_props import FileProp
 from gajim.common.const import JingleState
 from gajim.common.const import KindConstant
+from gajim.common.file_props import FileProp
 from gajim.common.helpers import AdditionalDataDict
 
 if typing.TYPE_CHECKING:
     from gajim.common.client import Client
     from gajim.common.modules.httpupload import HTTPFileTransfer
-    from gajim.plugins.pluginmanager import PluginManifest
 
 ChatListEventT = Union[
     'MessageReceived',
@@ -407,7 +406,7 @@ class MessageReceived(ApplicationEvent):
     unique_id: str
     stanza_id: str
     fjid: str
-    resource: str
+    resource: str | None
     delayed: Optional[float]
     msg_log_id: Optional[int]
     displaymarking: Optional[Displaymarking]
@@ -451,18 +450,6 @@ class RosterPush(ApplicationEvent):
     item: RosterItem
 
 
-@dataclass
-class PluginAdded(ApplicationEvent):
-    name: str = field(init=False, default='plugin-added')
-    manifest: PluginManifest
-
-
-@dataclass
-class PluginRemoved(ApplicationEvent):
-    name: str = field(init=False, default='plugin-removed')
-    manifest: PluginManifest
-
-
 @dataclass
 class SearchFormReceivedEvent(ApplicationEvent):
     name: str = field(init=False, default='search-form-received')
@@ -616,7 +603,7 @@ class SecCatalogReceived(ApplicationEvent):
     name: str = field(init=False, default='sec-catalog-received')
     account: str
     jid: str
-    catalog: dict[str, Catalog]
+    catalog: Catalog
 
 
 @dataclass
