@@ -27,12 +27,12 @@ class _DEFAULT:
     pass
 
 
-class _ACCOUNT_DEFAULT:
+class _ACCOUNTDEFAULT:
     pass
 
 
 HAS_APP_DEFAULT = _DEFAULT()
-HAS_ACCOUNT_DEFAULT = _ACCOUNT_DEFAULT()
+HAS_ACCOUNT_DEFAULT = _ACCOUNTDEFAULT()
 
 
 BoolSettings = Literal[
@@ -51,6 +51,7 @@ BoolSettings = Literal[
     'dev_force_bookmark_2',
     'developer_modus',
     'enable_emoji_shortcodes',
+    'enable_keepassxc_integration',
     'enable_negative_priority',
     'enable_file_preview',
     'escape_key_closes',
@@ -193,6 +194,7 @@ APP_SETTINGS = {
     'developer_modus': False,
     'dictionary_url': 'WIKTIONARY',
     'enable_emoji_shortcodes': True,
+    'enable_keepassxc_integration': False,
     'enable_negative_priority': False,
     'enable_file_preview': True,
     'escape_key_closes': False,
