@@ -15,49 +15,50 @@
 from __future__ import annotations
 
 from typing import Any
+from typing import cast
 from typing import Optional
 from typing import Union
-from typing import cast
 
-import os
 import logging
 import time
 
 from gi.repository import Gio
-from gi.repository import Gtk
 from gi.repository import GLib
-
+from gi.repository import Gtk
 from nbxmpp import JID
 from nbxmpp.const import StatusCode
-from nbxmpp.structs import MucSubject
 from nbxmpp.modules.security_labels import Displaymarking
+from nbxmpp.structs import MucSubject
 
 from gajim.common import app
 from gajim.common import events
 from gajim.common import ged
 from gajim.common import helpers
 from gajim.common import types
-from gajim.common.helpers import get_file_path_from_dnd_dropped_uri
-from gajim.common.helpers import to_user_string
-from gajim.common.i18n import _
+from gajim.common.const import KindConstant
 from gajim.common.ged import EventHelper
 from gajim.common.helpers import AdditionalDataDict
 from gajim.common.helpers import get_retraction_text
-from gajim.common.const import KindConstant
+from gajim.common.helpers import to_user_string
+from gajim.common.i18n import _
 from gajim.common.modules.contacts import BareContact
-from gajim.common.modules.contacts import GroupchatParticipant
 from gajim.common.modules.contacts import GroupchatContact
+from gajim.common.modules.contacts import GroupchatParticipant
 from gajim.common.modules.httpupload import HTTPFileTransfer
 from gajim.common.storage.archive import ConversationRow
 
-from gajim.gui.conversation.message_selection import MessageSelection
-from gajim.gui.conversation.view import ConversationView
-from gajim.gui.conversation.jump_to_end_button import JumpToEndButton
-from gajim.gui.builder import get_builder
-from gajim.gui.groupchat_roster import GroupchatRoster
-from gajim.gui.groupchat_state import GroupchatState
+from gajim.gtk.builder import get_builder
+from gajim.gtk.conversation.jump_to_end_button import JumpToEndButton
+from gajim.gtk.conversation.message_selection import MessageSelection
+from gajim.gtk.conversation.view import ConversationView
+from gajim.gtk.groupchat_roster import GroupchatRoster
+from gajim.gtk.groupchat_state import GroupchatState
 
-log = logging.getLogger('gajim.gui.control')
+HistoryRowT = Union[events.ApplicationEvent, ConversationRow]
+
+REQUEST_LINES_COUNT = 20
+
+log = logging.getLogger('gajim.gtk.control')
 
 
 class ChatControl(EventHelper):
@@ -65,7 +66,6 @@ class ChatControl(EventHelper):
         EventHelper.__init__(self)
 
         self.handlers: dict[int, Any] = {}
-
         self._contact = None
         self._client = None
 
@@ -74,8 +74,7 @@ class ChatControl(EventHelper):
         self._scrolled_view = ConversationView()
         self._scrolled_view.connect('autoscroll-changed',
                                     self._on_autoscroll_changed)
-        self._scrolled_view.connect('request-history',
-                                    self._fetch_n_lines_history, 20)
+        self._scrolled_view.connect('request-history', self._request_history)
         self._ui.conv_view_overlay.add(self._scrolled_view)
 
         self._groupchat_state = GroupchatState()
@@ -232,6 +231,8 @@ class ChatControl(EventHelper):
              ged.GUI2, self._on_jingle_request_received),
             ('file-request-received', ged.GUI2, self._on_file_request_event),
             ('file-request-sent', ged.GUI2, self._on_file_request_event),
+            ('http-upload-started', ged.GUI2, self._on_http_upload_started),
+            ('http-upload-error', ged.GUI2, self._on_http_upload_error),
         ])
 
     def _is_event_processable(self, event: Any) -> bool:
@@ -286,6 +287,7 @@ class ChatControl(EventHelper):
                          tim=event.timestamp,
                          displaymarking=displaymarking,
                          message_id=message_id,
+                         msg_log_id=event.msg_log_id,
                          additional_data=event.additional_data)
 
     def _on_message_received(self, event: events.MessageReceived) -> None:
@@ -367,6 +369,7 @@ class ChatControl(EventHelper):
                                  displaymarking=event.displaymarking,
                                  message_id=event.properties.id,
                                  stanza_id=event.stanza_id,
+                                 msg_log_id=event.msg_log_id,
                                  additional_data=event.additional_data)
 
     def _on_message_updated(self, event: events.MessageUpdated) -> None:
@@ -437,6 +440,12 @@ class ChatControl(EventHelper):
 
         self.add_jingle_file_transfer(event=event)
 
+    def _on_http_upload_started(self, event: events.HTTPUploadStarted) -> None:
+        self.add_file_transfer(event.transfer)
+
+    def _on_http_upload_error(self, event: events.HTTPUploadError) -> None:
+        self.add_info_message(event.error_msg)
+
     @property
     def is_chat(self) -> bool:
         return isinstance(self.contact, BareContact)
@@ -449,7 +458,7 @@ class ChatControl(EventHelper):
     def is_groupchat(self) -> bool:
         return isinstance(self.contact, GroupchatContact)
 
-    def mark_as_read(self, send_marker: bool = True) -> None:
+    def mark_as_read(self) -> None:
         self._jump_to_end_button.reset_unread_count()
 
     def _on_autoscroll_changed(self,
@@ -485,17 +494,8 @@ class ChatControl(EventHelper):
         self.reset_view()
 
     def drag_data_file_transfer(self, selection: Gtk.SelectionData) -> None:
-        # we may have more than one file dropped
-        uri_splitted = selection.get_uris()
-        for uri in uri_splitted:
-            path = get_file_path_from_dnd_dropped_uri(uri)
-            if not os.path.isfile(path):  # is it a file?
-                self.add_info_message(
-                    _('The following file could not be accessed and was '
-                      'not uploaded: %s') % path)
-                continue
-
-            app.interface.start_file_transfer(self.contact, path)
+        app.window.activate_action('send-file',
+                                   GLib.Variant('as', selection.get_uris()))
 
     def get_our_nick(self) -> str:
         if isinstance(self.contact, GroupchatParticipant):
@@ -592,14 +592,7 @@ class ChatControl(EventHelper):
             log_line_id)
         GLib.idle_add(self._scrolled_view.block_signals, False)
 
-    def _fetch_n_lines_history(self,
-                               _scrolled: Gtk.ScrolledWindow,
-                               before: bool,
-                               n_lines: int
-                               ) -> None:
-
-        self._scrolled_view.block_signals(True)
-
+    def _request_messages(self, before: bool) -> list[ConversationRow]:
         if before:
             row = self._scrolled_view.get_first_message_row()
         else:
@@ -610,66 +603,107 @@ class ChatControl(EventHelper):
         else:
             timestamp = row.db_timestamp
 
-        assert self._contact is not None
-        for event in app.storage.events.load(self._contact, before, timestamp):
-            if isinstance(event, events.MUCUserJoined):
-                self._process_muc_user_joined(event)
-
-            elif isinstance(event, events.MUCUserLeft):
-                self._process_muc_user_left(event)
-
-            elif isinstance(event, events.MUCNicknameChanged):
-                self._process_muc_nickname_changed(event)
-
-            elif isinstance(event, events.MUCRoomKicked):
-                self._process_muc_room_kicked(event)
-
-            elif isinstance(event, events.MUCUserAffiliationChanged):
-                self._process_muc_user_affiliation_changed(event)
-
-            elif isinstance(event, events.MUCUserRoleChanged):
-                self._process_muc_user_role_changed(event)
-
-            elif isinstance(event, events.MUCUserStatusShowChanged):
-                self._process_muc_user_status_show_changed(event)
-
-            elif isinstance(event, events.MUCRoomConfigChanged):
-                self._process_muc_room_config_changed(event)
-
-            elif isinstance(event, events.MUCRoomConfigFinished):
-                self._process_muc_room_config_finished(event)
-
-            elif isinstance(event, events.MUCRoomPresenceError):
-                self._process_muc_room_presence_error(event)
-
-            elif isinstance(event, events.MUCRoomDestroyed):
-                self._process_muc_room_destroyed(event)
-            else:
-                raise ValueError('Unknown event: %s' % event)
-
-        messages = app.storage.archive.get_conversation_before_after(
+        return app.storage.archive.get_conversation_before_after(
             self.contact.account,
             self.contact.jid,
             before,
             timestamp,
-            n_lines)
+            REQUEST_LINES_COUNT)
 
-        if not messages:
-            self._scrolled_view.set_history_complete(before, True)
-            self._scrolled_view.block_signals(False)
-            return
+    def _request_events(self, before: bool) -> list[events.ApplicationEvent]:
+        if before:
+            row = self._scrolled_view.get_first_event_row()
+        else:
+            row = self._scrolled_view.get_last_event_row()
 
-        self.add_messages(messages)
+        if row is None:
+            timestamp = time.time()
+        else:
+            timestamp = row.db_timestamp
 
-        if len(messages) < n_lines:
+        assert self._contact is not None
+        return app.storage.events.load(self._contact,
+                                       before,
+                                       timestamp,
+                                       REQUEST_LINES_COUNT)
+
+    def _request_history(self,
+                         _scrolled: Gtk.ScrolledWindow,
+                         before: bool
+                         ) -> None:
+
+        self._scrolled_view.block_signals(True)
+
+        messages = self._request_messages(before)
+        event_rows = self._request_events(before)
+        rows = self._sort_request_rows(messages, event_rows, before)
+
+        assert self._contact is not None
+        for row in rows:
+            if not isinstance(row, events.ApplicationEvent):
+                self.add_messages([row])
+
+            elif isinstance(row, events.MUCUserJoined):
+                self._process_muc_user_joined(row)
+
+            elif isinstance(row, events.MUCUserLeft):
+                self._process_muc_user_left(row)
+
+            elif isinstance(row, events.MUCNicknameChanged):
+                self._process_muc_nickname_changed(row)
+
+            elif isinstance(row, events.MUCRoomKicked):
+                self._process_muc_room_kicked(row)
+
+            elif isinstance(row, events.MUCUserAffiliationChanged):
+                self._process_muc_user_affiliation_changed(row)
+
+            elif isinstance(row, events.MUCUserRoleChanged):
+                self._process_muc_user_role_changed(row)
+
+            elif isinstance(row, events.MUCUserStatusShowChanged):
+                self._process_muc_user_status_show_changed(row)
+
+            elif isinstance(row, events.MUCRoomConfigChanged):
+                self._process_muc_room_config_changed(row)
+
+            elif isinstance(row, events.MUCRoomConfigFinished):
+                self._process_muc_room_config_finished(row)
+
+            elif isinstance(row, events.MUCRoomPresenceError):
+                self._process_muc_room_presence_error(row)
+
+            elif isinstance(row, events.MUCRoomDestroyed):
+                self._process_muc_room_destroyed(row)
+
+            else:
+                raise ValueError('Unknown event: %s' % type(row))
+
+        if len(rows) < REQUEST_LINES_COUNT:
             self._scrolled_view.set_history_complete(before, True)
 
         self._scrolled_view.block_signals(False)
 
+    @staticmethod
+    def _sort_request_rows(messages: list[ConversationRow],
+                           event_rows: list[events.ApplicationEvent],
+                           before: bool
+                           ) -> list[HistoryRowT]:
+
+        def sort_func(obj: HistoryRowT) -> float:
+            if isinstance(obj, events.ApplicationEvent):
+                return obj.timestamp  # pyright: ignore
+            return obj.time
+
+        rows = messages + event_rows
+        rows.sort(key=sort_func, reverse=before)
+        return rows
+
     def add_messages(self, messages: list[ConversationRow]):
         for msg in messages:
             if msg.kind in (KindConstant.FILE_TRANSFER_INCOMING,
                             KindConstant.FILE_TRANSFER_OUTGOING):
+                assert msg.additional_data is not None
                 if msg.additional_data.get_value('gajim', 'type') == 'jingle':
                     self._scrolled_view.add_jingle_file_transfer(
                         db_message=msg)
@@ -1045,7 +1079,7 @@ class ChatControl(EventHelper):
         reason = event.reason
         reason = '' if reason is None else f': {reason}'
 
-        message = _('Group chat has been destroyed')
+        message = _('Group chat has been destroyed%s') % reason
 
         if event.alternate is not None:
             message += '\n' + _('You can join this group chat instead: '
@@ -1156,6 +1190,7 @@ class ChatControl(EventHelper):
                         displaymarking: Optional[Displaymarking] = None,
                         message_id: Optional[str] = None,
                         stanza_id: Optional[str] = None,
+                        msg_log_id: Optional[int] = None,
                         additional_data: Optional[AdditionalDataDict] = None,
                         ) -> None:
 
@@ -1174,6 +1209,7 @@ class ChatControl(EventHelper):
                           displaymarking=displaymarking,
                           message_id=message_id,
                           stanza_id=stanza_id,
+                          msg_log_id=msg_log_id,
                           additional_data=additional_data)
 
     def _on_room_subject(self,
