@@ -14,8 +14,8 @@
 
 from __future__ import annotations
 
-from typing import Union
 from typing import Optional
+from typing import Union
 
 import logging
 import time
@@ -30,32 +30,33 @@ from gajim.common import app
 from gajim.common import ged
 from gajim.common.const import AvatarSize
 from gajim.common.const import KindConstant
-from gajim.common.events import FileRequestReceivedEvent
-from gajim.common.events import FileRequestSent
 from gajim.common.events import FileCompleted
-from gajim.common.events import FileProgress
 from gajim.common.events import FileError
 from gajim.common.events import FileHashError
-from gajim.common.events import FileSendError
+from gajim.common.events import FileProgress
 from gajim.common.events import FileRequestError
+from gajim.common.events import FileRequestReceivedEvent
+from gajim.common.events import FileRequestSent
+from gajim.common.events import FileSendError
 from gajim.common.events import JingleErrorReceived
 from gajim.common.events import JingleFtCancelledReceived
 from gajim.common.file_props import FileProp
 from gajim.common.file_props import FilesProp
 from gajim.common.helpers import open_file
+from gajim.common.helpers import show_in_folder
 from gajim.common.i18n import _
 from gajim.common.modules.contacts import BareContact
 from gajim.common.storage.archive import ConversationRow
 
-from .base import BaseRow
-from .widgets import DateTimeLabel
-from .widgets import NicknameLabel
-from ...builder import get_builder
-from ...util import format_eta
+from gajim.gtk.builder import get_builder
+from gajim.gtk.conversation.rows.base import BaseRow
+from gajim.gtk.conversation.rows.widgets import DateTimeLabel
+from gajim.gtk.conversation.rows.widgets import NicknameLabel
+from gajim.gtk.util import format_eta
 
 TransferEventT = Union[FileRequestReceivedEvent, FileRequestSent]
 
-log = logging.getLogger('gajim.gui.conversation.rows.file_transfer_jingle')
+log = logging.getLogger('gajim.gtk.conversation.rows.file_transfer_jingle')
 
 
 class FileTransferJingleRow(BaseRow):
@@ -79,6 +80,7 @@ class FileTransferJingleRow(BaseRow):
         self._contact = contact
 
         if db_message is not None:
+            assert db_message.additional_data is not None
             sid = db_message.additional_data.get_value('gajim', 'sid')
             assert sid is not None
             self._file_props = FilesProp.getFilePropBySid(sid)
@@ -352,9 +354,7 @@ class FileTransferJingleRow(BaseRow):
     def _on_open_folder(self, _button: Gtk.Button) -> None:
         assert self._file_props is not None
         assert self._file_props.file_name is not None
-
-        folder = Path(self._file_props.file_name).parent
-        open_file(folder)
+        show_in_folder(Path(self._file_props.file_name))
 
     def _on_bad_hash_retry(self, _button: Gtk.Button) -> None:
         app.interface.instances['file_transfers'].show_hash_error(
