@@ -1,48 +1,70 @@
+#
+# This file is part of Gajim.
+#
+# Gajim is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published
+# by the Free Software Foundation; version 3 only.
+#
+# Gajim is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+
+from typing import Any
+from typing import cast
+from typing import Optional
+from typing import Union
+
 import logging
 
-from gi.repository import Gio
+import cairo
 from gi.repository import Gdk
-from gi.repository import Gtk
+from gi.repository import Gio
 from gi.repository import GLib
-
+from gi.repository import Gtk
 from nbxmpp.errors import StanzaError
-from nbxmpp.namespaces import Namespace
-from nbxmpp.modules.vcard4 import VCard
 from nbxmpp.modules.user_avatar import Avatar
+from nbxmpp.modules.vcard4 import VCard
+from nbxmpp.namespaces import Namespace
+from nbxmpp.task import Task
 
 from gajim.common import app
 from gajim.common.const import AvatarSize
 from gajim.common.i18n import _
-from gajim.common.i18n import Q_
+from gajim.common.i18n import p_
 
-from gajim.gui.avatar import clip_circle
-from gajim.gui.avatar_selector import AvatarSelector
-from gajim.gui.dialogs import ErrorDialog
-from gajim.gui.filechoosers import AvatarChooserDialog
-from gajim.gui.util import get_builder
-from gajim.gui.vcard_grid import VCardGrid
-from gajim.gui.util import scroll_to_end
+from gajim.gtk.avatar import clip_circle
+from gajim.gtk.avatar_selector import AvatarSelector
+from gajim.gtk.builder import get_builder
+from gajim.gtk.dialogs import ErrorDialog
+from gajim.gtk.filechoosers import AvatarChooserDialog
+from gajim.gtk.util import scroll_to_end
+from gajim.gtk.vcard_grid import VCardGrid
 
-log = logging.getLogger('gajim.gui.profile')
+log = logging.getLogger('gajim.gtk.profile')
 
 MENU_DICT = {
-    'fn': Q_('?profile:Full Name'),
+    'fn': p_('Profile', 'Full Name'),
     'bday': _('Birthday'),
-    'gender': Q_('?profile:Gender'),
-    'adr': Q_('?profile:Address'),
+    'gender': p_('Profile', 'Gender'),
+    'adr': p_('Profile', 'Address'),
     'email': _('Email'),
-    'impp': Q_('?profile:IM Address'),
+    'impp': p_('Profile', 'IM Address'),
     'tel': _('Phone No.'),
-    'org': Q_('?profile:Organisation'),
-    'title': Q_('?profile:Title'),
-    'role': Q_('?profile:Role'),
+    'org': p_('Profile', 'Organisation'),
+    'title': p_('Profile', 'Title'),
+    'role': p_('Profile', 'Role'),
     'url': _('URL'),
-    'key': Q_('?profile:Public Encryption Key'),
+    'key': p_('Profile', 'Public Encryption Key'),
+    'note': p_('Profile', 'Note'),
 }
 
 
 class ProfileWindow(Gtk.ApplicationWindow):
-    def __init__(self, account, *args):
+    def __init__(self, account: str) -> None:
         Gtk.ApplicationWindow.__init__(self)
         self.set_application(app.app)
         self.set_position(Gtk.WindowPosition.CENTER)
@@ -56,6 +78,10 @@ class ProfileWindow(Gtk.ApplicationWindow):
         self.account = account
         self._jid = app.get_jid_from_account(account)
 
+        self._client = app.get_client(self.account)
+        self._contact = self._client.get_module('Contacts').get_contact(
+            self._jid)
+
         self._ui = get_builder('profile.ui')
 
         menu = Gio.Menu()
@@ -65,15 +91,15 @@ class ProfileWindow(Gtk.ApplicationWindow):
         self._ui.add_entry_button.set_menu_model(menu)
         self._add_actions()
 
-        self._avatar_selector = None
-        self._current_avatar = None
-        self._current_vcard = None
-        self._avatar_nick_public = None
+        self._avatar_selector: Optional[AvatarSelector] = None
+        self._current_avatar: Optional[cairo.ImageSurface] = None
+        self._current_vcard: Optional[VCard] = None
+        self._avatar_nick_public: Optional[bool] = None
 
         # False  - no change to avatar
         # None   - we want to delete the avatar
         # Avatar - upload new avatar
-        self._new_avatar = False
+        self._new_avatar: Union[None, bool, Avatar] = False
 
         self._ui.nickname_entry.set_text(app.nicks[account])
 
@@ -110,9 +136,16 @@ class ProfileWindow(Gtk.ApplicationWindow):
             user_data=Namespace.NICK)
 
         self._ui.connect_signals(self)
-        self.connect('key-press-event', self._on_key_press_event)
+        self.connect('key-press-event', self._on_key_press)
 
-    def _on_access_model_received(self, task):
+        self.connect('destroy', self._on_destroy)
+
+    def _on_destroy(self, *args: Any) -> None:
+        self._avatar_selector = None
+        self._ui.privacy_popover.destroy()
+        app.check_finalize(self)
+
+    def _on_access_model_received(self, task: Task) -> None:
         namespace = task.get_user_data()
 
         try:
@@ -125,59 +158,57 @@ class ProfileWindow(Gtk.ApplicationWindow):
         access_model = result == 'open'
 
         if namespace == Namespace.VCARD4_PUBSUB:
-            self._ui.vcard_access.set_active(access_model)
+            self._set_vcard_access_switch(access_model)
         else:
             if self._avatar_nick_public is None:
                 self._avatar_nick_public = access_model
             else:
-                self._avatar_nick_public = (self._avatar_nick_public and
+                self._avatar_nick_public = (self._avatar_nick_public or
                                             access_model)
-            self._ui.avatar_nick_access.set_active(self._avatar_nick_public)
+            assert self._avatar_nick_public is not None
+            self._set_avatar_nick_access_switch(self._avatar_nick_public)
 
-    def _on_vcard_received(self, task):
+    def _on_vcard_received(self, task: Task):
         try:
-            self._current_vcard = task.finish()
+            self._current_vcard = cast(VCard, task.finish())
         except StanzaError as error:
             log.info('Error loading VCard: %s', error)
             self._current_vcard = VCard()
 
-        if self._current_vcard is None:
-            self._current_vcard = VCard()
-
         self._load_avatar()
         self._vcard_grid.set_vcard(self._current_vcard.copy())
         self._ui.profile_stack.set_visible_child_name('profile')
         self._ui.spinner.stop()
 
-    def _load_avatar(self):
+    def _load_avatar(self) -> None:
         scale = self.get_scale_factor()
-        self._current_avatar = app.contacts.get_avatar(
-            self.account,
-            self._jid,
-            AvatarSize.VCARD,
-            scale)
-
+        self._current_avatar = self._contact.get_avatar(AvatarSize.VCARD,
+                                                        scale,
+                                                        add_show=False)
         self._ui.avatar_image.set_from_surface(self._current_avatar)
         self._ui.avatar_image.show()
 
-    def _on_key_press_event(self, _widget, event):
+    def _on_key_press(self, _widget: Gtk.Widget, event: Gdk.EventKey) -> None:
         if event.keyval == Gdk.KEY_Escape:
             self.destroy()
 
-    def _add_actions(self):
+    def _add_actions(self) -> None:
         for action in MENU_DICT:
             action_name = 'add-' + action.lower()
             act = Gio.SimpleAction.new(action_name, None)
             act.connect('activate', self._on_action)
             self.add_action(act)
 
-    def _on_action(self, action, _param):
+    def _on_action(self,
+                   action: Gio.SimpleAction,
+                   _param: Optional[GLib.Variant]
+                   ) -> None:
         name = action.get_name()
         key = name.split('-')[1]
         self._vcard_grid.add_new_property(key)
         GLib.idle_add(scroll_to_end, self._ui.scrolled)
 
-    def _on_edit_clicked(self, *args):
+    def _on_edit_clicked(self, *args: Any) -> None:
         self._vcard_grid.set_editable(True)
         self._ui.edit_button.hide()
         self._ui.add_entry_button.set_no_show_all(False)
@@ -189,7 +220,7 @@ class ProfileWindow(Gtk.ApplicationWindow):
         self._ui.nickname_entry.set_sensitive(True)
         self._ui.privacy_button.show()
 
-    def _on_cancel_clicked(self, _widget):
+    def _on_cancel_clicked(self, _button: Gtk.Button) -> None:
         self._vcard_grid.set_editable(False)
         self._ui.edit_button.show()
         self._ui.add_entry_button.hide()
@@ -201,10 +232,10 @@ class ProfileWindow(Gtk.ApplicationWindow):
         self._ui.nickname_entry.set_sensitive(False)
         self._ui.avatar_image.set_from_surface(self._current_avatar)
         self._ui.nickname_entry.set_text(app.nicks[self.account])
-        self._vcard_grid.set_vcard(self._current_vcard.copy())
+        self._vcard_grid.set_vcard(self._current_vcard.copy())  # pyright: ignore # noqa: E501
         self._new_avatar = False
 
-    def _on_save_clicked(self, _widget):
+    def _on_save_clicked(self, _button: Gtk.Button) -> None:
         self._ui.spinner.start()
         self._ui.profile_stack.set_visible_child_name('spinner')
         self._ui.add_entry_button.hide()
@@ -220,10 +251,10 @@ class ProfileWindow(Gtk.ApplicationWindow):
         self._vcard_grid.sort()
 
         vcard = self._vcard_grid.get_vcard()
-        self._current_vcard = vcard.copy()
+        self._current_vcard = vcard.copy()  # pyright: ignore
 
-        con = app.connections[self.account]
-        con.get_module('VCard4').set_vcard(
+        client = app.get_client(self.account)
+        client.get_module('VCard4').set_vcard(
             self._current_vcard,
             public=self._ui.vcard_access.get_active(),
             callback=self._on_save_finished)
@@ -232,24 +263,24 @@ class ProfileWindow(Gtk.ApplicationWindow):
 
         if self._new_avatar is False:
             if self._avatar_nick_public != public:
-                con.get_module('UserAvatar').set_access_model(public)
+                client.get_module('UserAvatar').set_access_model(public)
 
         else:
             # Only update avatar if it changed
-            con.get_module('UserAvatar').set_avatar(
+            client.get_module('UserAvatar').set_avatar(
                 self._new_avatar,
                 public=public,
                 callback=self._on_set_avatar)
 
         nick = GLib.markup_escape_text(self._ui.nickname_entry.get_text())
-        con.get_module('UserNickname').set_nickname(nick, public=public)
+        client.get_module('UserNickname').set_nickname(nick, public=public)
 
         if not nick:
             nick = app.settings.get_account_setting(
                 self.account, 'name')
         app.nicks[self.account] = nick
 
-    def _on_set_avatar(self, task):
+    def _on_set_avatar(self, task: Task) -> None:
         try:
             task.finish()
         except StanzaError as error:
@@ -271,34 +302,37 @@ class ProfileWindow(Gtk.ApplicationWindow):
             self._new_avatar = False
             return
 
-    def _on_remove_avatar(self, _button):
-        contact = app.contacts.create_contact(self._jid, self.account)
+    def _on_remove_avatar(self, _button: Gtk.Button) -> None:
         scale = self.get_scale_factor()
-        surface = app.interface.avatar_storage.get_surface(
-            contact, AvatarSize.VCARD, scale, default=True)
+        surface = self._contact.get_avatar(AvatarSize.VCARD,
+                                           scale,
+                                           add_show=False,
+                                           default=True)
 
         self._ui.avatar_image.set_from_surface(surface)
         self._ui.remove_avatar_button.hide()
         self._new_avatar = None
 
-    def _on_edit_avatar(self, button):
-        def _on_file_selected(path):
+    def _on_edit_avatar(self, button: Gtk.Button) -> None:
+        def _on_file_selected(paths: list[str]) -> None:
             if self._avatar_selector is None:
                 self._avatar_selector = AvatarSelector()
                 self._ui.avatar_selector_box.add(self._avatar_selector)
 
-            self._avatar_selector.prepare_crop_area(path)
+            self._avatar_selector.prepare_crop_area(paths[0])
             self._ui.avatar_update_button.set_sensitive(
                 self._avatar_selector.get_prepared())
             self._ui.profile_stack.set_visible_child_name('avatar_selector')
 
         AvatarChooserDialog(_on_file_selected,
-                            transient_for=button.get_toplevel())
+                            transient_for=cast(Gtk.Window,
+                                               button.get_toplevel()))
 
-    def _on_cancel_update_avatar(self, _button):
+    def _on_cancel_update_avatar(self, _button: Gtk.Button) -> None:
         self._ui.profile_stack.set_visible_child_name('profile')
 
-    def _on_update_avatar(self, _button):
+    def _on_update_avatar(self, _button: Gtk.Button) -> None:
+        assert self._avatar_selector
         success, data, width, height = self._avatar_selector.get_avatar_bytes()
         if not success:
             self._ui.profile_stack.set_visible_child_name('profile')
@@ -306,7 +340,10 @@ class ProfileWindow(Gtk.ApplicationWindow):
                         _('Failed to generate avatar.'))
             return
 
-        sha = app.interface.avatar_storage.save_avatar(data)
+        assert data
+        assert height
+        assert width
+        sha = app.app.avatar_storage.save_avatar(data)
         if sha is None:
             self._ui.profile_stack.set_visible_child_name('profile')
             ErrorDialog(_('Error while processing image'),
@@ -314,25 +351,36 @@ class ProfileWindow(Gtk.ApplicationWindow):
             return
 
         self._new_avatar = Avatar()
-        self._new_avatar.add_image_source(data, 'image/png', height, width)
+        self._new_avatar.add_image_source(data, 'image/png', height, width)  # pyright: ignore  # noqa: E501
 
         scale = self.get_scale_factor()
-        surface = app.interface.avatar_storage.surface_from_filename(
+        surface = app.app.avatar_storage.surface_from_filename(
             sha, AvatarSize.VCARD, scale)
+        if surface is None:
+            return
 
         self._ui.avatar_image.set_from_surface(clip_circle(surface))
         self._ui.remove_avatar_button.show()
         self._ui.profile_stack.set_visible_child_name('profile')
 
-    def _access_switch_toggled(self, *args):
-        avatar_nick_access = self._ui.avatar_nick_access.get_active()
-        vcard_access = self._ui.vcard_access.get_active()
-        self._ui.avatar_nick_access_label.set_text(
-            _('Everyone') if avatar_nick_access else _('Contacts'))
+    def _set_vcard_access_switch(self, state: bool) -> None:
+        self._ui.vcard_access.set_active(state)
         self._ui.vcard_access_label.set_text(
-            _('Everyone') if vcard_access else _('Contacts'))
+            _('Everyone') if state else _('Contacts'))
 
-    def _on_save_finished(self, task):
+    def _set_avatar_nick_access_switch(self, state: bool) -> None:
+        self._ui.avatar_nick_access.set_active(state)
+        self._ui.avatar_nick_access_label.set_text(
+            _('Everyone') if state else _('Contacts'))
+
+    def _access_switch_toggled(self, *args: Any) -> None:
+        state = self._ui.vcard_access.get_active()
+        self._set_vcard_access_switch(state)
+
+        state = self._ui.avatar_nick_access.get_active()
+        self._set_avatar_nick_access_switch(state)
+
+    def _on_save_finished(self, task: Task) -> None:
         try:
             task.finish()
         except StanzaError as err:
